@@ -1,5 +1,5 @@
 use std::convert::TryFrom;
-use std::io::Write;
+use std::io::{BufRead, Write};
 use std::str::FromStr;
 
 use clap::ArgMatches;
@@ -9,12 +9,12 @@ use regex::Regex;
 use crate::error::{Result, Error};
 use crate::database::Database;
 use crate::formatters::Formatter;
-use crate::config::Config;
 use crate::timeparse::parse_time;
 use crate::regex::parse_regex;
 use crate::old::{entries_or_warning, time_or_warning, warn_if_needed};
+use crate::io::Streams;
 
-use super::Command;
+use super::{Command, Facts};
 
 // ----------------------------------------------------------------
 // Things that are general to all commands that display in some way
@@ -39,28 +39,29 @@ impl FromStr for Sheet {
 }
 
 #[allow(clippy::too_many_arguments)]
-pub fn entries_for_display<D, O, E>(
+pub fn entries_for_display<D, I, O, E>(
     start: Option<DateTime<Utc>>, end: Option<DateTime<Utc>>,
-    sheet: Option<Sheet>, db: &mut D, out: &mut O, err: &mut E,
+    sheet: Option<Sheet>, streams: &mut Streams<D, I, O, E>,
     format: Formatter, ids: bool, grep: Option<Regex>,
-    now: DateTime<Utc>,
+    facts: &Facts,
 ) -> Result<()>
 where
     D: Database,
+    I: BufRead,
     O: Write,
     E: Write,
 {
-    let start = start.map(|s| time_or_warning(s, db)).transpose()?.map(|s| s.0);
-    let end = end.map(|e| time_or_warning(e, db)).transpose()?.map(|e| e.0);
+    let start = start.map(|s| time_or_warning(s, &streams.db)).transpose()?.map(|s| s.0);
+    let end = end.map(|e| time_or_warning(e, &streams.db)).transpose()?.map(|e| e.0);
 
     let mut entries = match sheet {
-        Some(Sheet::All) => db.entries_all_visible(start, end)?,
-        Some(Sheet::Full) => db.entries_full(start, end)?,
-        Some(Sheet::Sheet(name)) => db.entries_by_sheet(&name, start, end)?,
+        Some(Sheet::All) => streams.db.entries_all_visible(start, end)?,
+        Some(Sheet::Full) => streams.db.entries_full(start, end)?,
+        Some(Sheet::Sheet(name)) => streams.db.entries_by_sheet(&name, start, end)?,
         None => {
-            let current_sheet = db.current_sheet()?.unwrap_or_else(|| "default".into());
+            let current_sheet = streams.db.current_sheet()?.unwrap_or_else(|| "default".into());
 
-            db.entries_by_sheet(&current_sheet, start, end)?
+            streams.db.entries_by_sheet(&current_sheet, start, end)?
         }
     };
 
@@ -68,16 +69,16 @@ where
         entries.retain(|e| re.is_match(&e.note.clone().unwrap_or_else(String::new)));
     }
 
-    let (entries, needs_warning) = entries_or_warning(entries, db)?;
+    let (entries, needs_warning) = entries_or_warning(entries, &streams.db)?;
 
     format.print_formatted(
         entries,
-        out,
-        now,
+        &mut streams.out,
+        facts,
         ids,
     )?;
 
-    warn_if_needed(err, needs_warning)?;
+    warn_if_needed(&mut streams.err, needs_warning, &facts.env)?;
 
     Ok(())
 }
@@ -116,9 +117,10 @@ pub struct DisplayCommand { }
 impl<'a> Command<'a> for DisplayCommand {
     type Args = Args;
 
-    fn handle<D, O, E>(args: Self::Args, db: &mut D, out: &mut O, err: &mut E, config: &Config, now: DateTime<Utc>) -> Result<()>
+    fn handle<D, I, O, E>(args: Self::Args, streams: &mut Streams<D, I, O, E>, facts: &Facts) -> Result<()>
     where
         D: Database,
+        I: BufRead,
         O: Write,
         E: Write,
     {
@@ -126,13 +128,11 @@ impl<'a> Command<'a> for DisplayCommand {
             args.start,
             args.end,
             args.sheet,
-            db,
-            out,
-            err,
-            args.format.unwrap_or_else(|| config.default_formatter.clone()),
+            streams,
+            args.format.unwrap_or_else(|| facts.config.default_formatter.clone()),
             args.ids,
             args.grep,
-            now
+            facts
         )
     }
 }
@@ -140,26 +140,27 @@ impl<'a> Command<'a> for DisplayCommand {
 #[cfg(test)]
 mod tests {
     use chrono::TimeZone;
-    use ansi_term::Color::Yellow;
     use pretty_assertions::assert_eq;
 
     use crate::database::SqliteDatabase;
     use crate::test_utils::Ps;
+    use crate::config::Config;
 
     use super::*;
 
     #[test]
     fn display_as_local_time_if_previous_version() {
         std::env::set_var("TZ", "CST+6");
+
         let args = Default::default();
-        let mut db = SqliteDatabase::from_path("assets/test_old_db.db").unwrap();
-        let mut out = Vec::new();
-        let mut err = Vec::new();
-        let config = Default::default();
+        let mut streams = Streams::fake(b"").with_db(
+            SqliteDatabase::from_path("assets/test_old_db.db").unwrap()
+        );
+        let facts = Facts::new();
 
-        DisplayCommand::handle(args, &mut db, &mut out, &mut err, &config, Utc::now()).unwrap();
+        DisplayCommand::handle(args, &mut streams, &facts).unwrap();
 
-        assert_eq!(Ps(&String::from_utf8_lossy(&out)), Ps("Timesheet: default
+        assert_eq!(Ps(&String::from_utf8_lossy(&streams.out)), Ps("Timesheet: default
     Day                Start      End        Duration Notes
     Tue Jun 29, 2021   06:26:49 - 07:26:52    1:00:03 lets do some rust
                                               1:00:03
@@ -168,8 +169,8 @@ mod tests {
 "));
 
         assert_eq!(
-            String::from_utf8_lossy(&err),
-            format!("{} You are using the old timetrap format, it is advised that you update your database using t migrate\n", Yellow.bold().paint("[WARNING]")),
+            String::from_utf8_lossy(&streams.err),
+            "[WARNING] You are using the old timetrap format, it is advised that you update your database using t migrate\n"
         );
     }
 
@@ -180,47 +181,40 @@ mod tests {
             start: Some(Utc.ymd(2021, 6, 30).and_hms(10, 5, 0)),
             ..Default::default()
         };
-        let mut db = SqliteDatabase::from_memory().unwrap();
-        let mut out = Vec::new();
-        let mut err = Vec::new();
-        let config = Default::default();
+        let mut streams = Streams::fake(b"");
+        let facts = Facts::new();
 
-        db.init().unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 0, 0), None, Some("hola".into()), "default".into()).unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 10, 0), None, Some("hola".into()), "default".into()).unwrap();
 
-        db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 0, 0), None, Some("hola".into()), "default".into()).unwrap();
-        db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 10, 0), None, Some("hola".into()), "default".into()).unwrap();
+        DisplayCommand::handle(args, &mut streams, &facts).unwrap();
 
-        DisplayCommand::handle(args, &mut db, &mut out, &mut err, &config, Utc::now()).unwrap();
-
-        assert_eq!(Ps(&String::from_utf8_lossy(&out)), Ps("start,end,note,sheet
+        assert_eq!(Ps(&String::from_utf8_lossy(&streams.out)), Ps("start,end,note,sheet
 2021-06-30T10:10:00.000000Z,,hola,default
 "));
 
         assert_eq!(
-            String::from_utf8_lossy(&err),
+            String::from_utf8_lossy(&streams.err),
             String::new(),
         );
     }
 
     #[test]
     fn filter_by_match() {
-        let mut db = SqliteDatabase::from_memory().unwrap();
-        let mut out = Vec::new();
-        let mut err = Vec::new();
+        let mut streams = Streams::fake(b"");
+        let facts = Facts::new();
 
-        db.init().unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 0, 0), None, Some("hola".into()), "default".into()).unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 10, 0), None, Some("adios".into()), "default".into()).unwrap();
 
-        db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 0, 0), None, Some("hola".into()), "default".into()).unwrap();
-        db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 10, 0), None, Some("adios".into()), "default".into()).unwrap();
+        entries_for_display(None, None, None, &mut streams, Formatter::Csv, true, Some("io".parse().unwrap()), &facts).unwrap();
 
-        entries_for_display(None, None, None, &mut db, &mut out, &mut err, Formatter::Csv, true, Some("io".parse().unwrap()), Utc::now()).unwrap();
-
-        assert_eq!(Ps(&String::from_utf8_lossy(&out)), Ps("id,start,end,note,sheet
+        assert_eq!(Ps(&String::from_utf8_lossy(&streams.out)), Ps("id,start,end,note,sheet
 2,2021-06-30T10:10:00.000000Z,,adios,default
 "));
 
         assert_eq!(
-            String::from_utf8_lossy(&err),
+            String::from_utf8_lossy(&streams.err),
             String::new(),
         );
     }
@@ -231,21 +225,17 @@ mod tests {
             sheet: Some(Sheet::All),
             ..Default::default()
         };
-        let mut db = SqliteDatabase::from_memory().unwrap();
-        let mut out = Vec::new();
-        let mut err = Vec::new();
-        let config = Default::default();
+        let mut streams = Streams::fake(b"");
+        let facts = Facts::new();
         std::env::set_var("TZ", "CST+6");
 
-        db.init().unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 0, 0), Some(Utc.ymd(2021, 6, 30).and_hms(11, 0, 0)), None, "sheet1".into()).unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(11, 0, 0), Some(Utc.ymd(2021, 6, 30).and_hms(12, 0, 0)), None, "sheet2".into()).unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(12, 0, 0), Some(Utc.ymd(2021, 6, 30).and_hms(13, 0, 0)), None, "sheet1".into()).unwrap();
 
-        db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 0, 0), Some(Utc.ymd(2021, 6, 30).and_hms(11, 0, 0)), None, "sheet1".into()).unwrap();
-        db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(11, 0, 0), Some(Utc.ymd(2021, 6, 30).and_hms(12, 0, 0)), None, "sheet2".into()).unwrap();
-        db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(12, 0, 0), Some(Utc.ymd(2021, 6, 30).and_hms(13, 0, 0)), None, "sheet1".into()).unwrap();
+        DisplayCommand::handle(args, &mut streams, &facts).unwrap();
 
-        DisplayCommand::handle(args, &mut db, &mut out, &mut err, &config, Utc::now()).unwrap();
-
-        assert_eq!(Ps(&String::from_utf8_lossy(&out)), Ps("Timesheet: sheet1
+        assert_eq!(Ps(&String::from_utf8_lossy(&streams.out)), Ps("Timesheet: sheet1
     Day                Start      End        Duration Notes
     Wed Jun 30, 2021   04:00:00 - 05:00:00    1:00:00
                        06:00:00 - 07:00:00    1:00:00
@@ -270,21 +260,17 @@ Timesheet: sheet2
             sheet: Some(Sheet::Full),
             ..Default::default()
         };
-        let mut db = SqliteDatabase::from_memory().unwrap();
-        let mut out = Vec::new();
-        let mut err = Vec::new();
-        let config = Default::default();
+        let mut streams = Streams::fake(b"");
+        let facts = Facts::new();
         std::env::set_var("TZ", "CST+6");
 
-        db.init().unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 0, 0), Some(Utc.ymd(2021, 6, 30).and_hms(11, 0, 0)), None, "sheet1".into()).unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(11, 0, 0), Some(Utc.ymd(2021, 6, 30).and_hms(12, 0, 0)), None, "_sheet2".into()).unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(12, 0, 0), Some(Utc.ymd(2021, 6, 30).and_hms(13, 0, 0)), None, "sheet1".into()).unwrap();
 
-        db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 0, 0), Some(Utc.ymd(2021, 6, 30).and_hms(11, 0, 0)), None, "sheet1".into()).unwrap();
-        db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(11, 0, 0), Some(Utc.ymd(2021, 6, 30).and_hms(12, 0, 0)), None, "_sheet2".into()).unwrap();
-        db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(12, 0, 0), Some(Utc.ymd(2021, 6, 30).and_hms(13, 0, 0)), None, "sheet1".into()).unwrap();
+        DisplayCommand::handle(args, &mut streams, &facts).unwrap();
 
-        DisplayCommand::handle(args, &mut db, &mut out, &mut err, &config, Utc::now()).unwrap();
-
-        assert_eq!(Ps(&String::from_utf8_lossy(&out)), Ps("\
+        assert_eq!(Ps(&String::from_utf8_lossy(&streams.out)), Ps("\
 Timesheet: _sheet2
     Day                Start      End        Duration Notes
     Wed Jun 30, 2021   05:00:00 - 06:00:00    1:00:00
@@ -314,24 +300,24 @@ Timesheet: sheet1
             end: Some(Utc.ymd(2021, 6, 29).and_hms(13, 0, 0)),
             ..Default::default()
         };
-        let mut db = SqliteDatabase::from_path("assets/test_old_db.db").unwrap();
-        let mut out = Vec::new();
-        let mut err = Vec::new();
-        let config = Default::default();
+        let mut streams = Streams::fake(b"").with_db(
+            SqliteDatabase::from_path("assets/test_old_db.db").unwrap()
+        );
+        let facts = Facts::new();
 
         // item in database:
         //     start: 2021-06-29 06:26:49.580565
         //     end:   2021-06-29 07:26:52.816747
 
-        DisplayCommand::handle(args, &mut db, &mut out, &mut err, &config, Utc::now()).unwrap();
+        DisplayCommand::handle(args, &mut streams, &facts).unwrap();
 
-        assert_eq!(Ps(&String::from_utf8_lossy(&out)), Ps("start,end,note,sheet
+        assert_eq!(Ps(&String::from_utf8_lossy(&streams.out)), Ps("start,end,note,sheet
 2021-06-29T12:26:49.580565Z,2021-06-29T13:26:52.816747Z,lets do some rust,default
 "));
 
         assert_eq!(
-            String::from_utf8_lossy(&err),
-            format!("{} You are using the old timetrap format, it is advised that you update your database using t migrate\n", Yellow.bold().paint("[WARNING]")),
+            String::from_utf8_lossy(&streams.err),
+            "[WARNING] You are using the old timetrap format, it is advised that you update your database using t migrate\n"
         );
     }
 
@@ -340,22 +326,18 @@ Timesheet: sheet1
         std::env::set_var("TZ", "CST+6");
 
         let args = Default::default();
-        let mut db = SqliteDatabase::from_memory().unwrap();
-        let mut out = Vec::new();
-        let mut err = Vec::new();
-        let config = Config {
+        let mut streams = Streams::fake(b"");
+        let facts = Facts::new().with_config(Config {
             default_formatter: Formatter::Ids,
             ..Default::default()
-        };
+        });
 
-        db.init().unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 0, 0), None, Some("hola".into()), "default".into()).unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 10, 0), None, Some("hola".into()), "default".into()).unwrap();
 
-        db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 0, 0), None, Some("hola".into()), "default".into()).unwrap();
-        db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 10, 0), None, Some("hola".into()), "default".into()).unwrap();
+        DisplayCommand::handle(args, &mut streams, &facts).unwrap();
 
-        DisplayCommand::handle(args, &mut db, &mut out, &mut err, &config, Utc::now()).unwrap();
-
-        assert_eq!(&String::from_utf8_lossy(&out), "1 2\n");
-        assert_eq!(String::from_utf8_lossy(&err), "");
+        assert_eq!(&String::from_utf8_lossy(&streams.out), "1 2\n");
+        assert_eq!(String::from_utf8_lossy(&streams.err), "");
     }
 }
