@@ -1,20 +1,20 @@
 use std::convert::TryFrom;
-use std::io::Write;
+use std::io::{BufRead, Write};
 
 use clap::ArgMatches;
-use chrono::{DateTime, Utc, Duration, Local};
+use chrono::{Utc, Duration, Local};
 use itertools::Itertools;
 use ansi_term::Style;
 
 use crate::error::{Error, Result};
 use crate::database::Database;
-use crate::config::Config;
 use crate::tabulate::{Tabulate, Col, Align::*};
 use crate::formatters::text::format_duration;
 use crate::models::Entry;
 use crate::old::{entries_or_warning, warn_if_needed};
+use crate::io::Streams;
 
-use super::Command;
+use super::{Command, Facts};
 
 #[derive(Default)]
 pub struct Args {
@@ -36,28 +36,29 @@ pub struct ListCommand {}
 impl<'a> Command<'a> for ListCommand {
     type Args = Args;
 
-    fn handle<D, O, E>(args: Args, db: &mut D, out: &mut O, err: &mut E, _config: &Config, now: DateTime<Utc>) -> Result<()>
+    fn handle<D, I, O, E>(args: Args, streams: &mut Streams<D, I, O, E>, facts: &Facts) -> Result<()>
     where
         D: Database,
+        I: BufRead,
         O: Write,
         E: Write,
     {
-        let today = now.with_timezone(&Local).date().and_hms(0, 0, 0).with_timezone(&Utc);
+        let today = facts.now.with_timezone(&Local).date().and_hms(0, 0, 0).with_timezone(&Utc);
         let entries = if args.all {
-            db.entries_full(None, None)?
+            streams.db.entries_full(None, None)?
         } else {
-            db.entries_all_visible(None, None)?
+            streams.db.entries_all_visible(None, None)?
         };
 
-        let (mut entries, needs_warning) = entries_or_warning(entries, db)?;
+        let (mut entries, needs_warning) = entries_or_warning(entries, &streams.db)?;
 
-        let current = db.current_sheet()?;
-        let last = db.last_sheet()?;
+        let current = streams.db.current_sheet()?;
+        let last = streams.db.last_sheet()?;
 
         // introducte two fake entries to make both current and last show up
         if let Some(ref current) = current {
             entries.push(Entry {
-                id: 1, sheet: current.clone(), start: now, end: Some(now), note: None,
+                id: 1, sheet: current.clone(), start: facts.now, end: Some(facts.now), note: None,
             });
         }
 
@@ -73,12 +74,12 @@ impl<'a> Command<'a> for ListCommand {
             .into_iter()
             .map(|(key, group)| {
                 let entries: Vec<_> = group.into_iter().collect();
-                let s_running = now - entries.iter().find(|e| e.end.is_none()).map(|e| e.start).unwrap_or(now);
+                let s_running = facts.now - entries.iter().find(|e| e.end.is_none()).map(|e| e.start).unwrap_or(facts.now);
                 let s_today = entries.iter().filter(|e| e.start > today).fold(Duration::seconds(0), |acc, e| {
-                    acc + (e.end.unwrap_or(now) - e.start)
+                    acc + (e.end.unwrap_or(facts.now) - e.start)
                 });
                 let s_total = entries.into_iter().fold(Duration::seconds(0), |acc, e| {
-                    acc + (e.end.unwrap_or(now) - e.start)
+                    acc + (e.end.unwrap_or(facts.now) - e.start)
                 });
 
                 total_running = total_running + s_running;
@@ -113,10 +114,14 @@ impl<'a> Command<'a> for ListCommand {
             Col::min_width(9).and_alignment(Left),
 
             // running time
-            Col::min_width(9).and_alignment(Right).color_if(Style::new().dimmed(), |s| s == "0:00:00"),
+            Col::min_width(9).and_alignment(Right)
+                .color_if(Style::new().dimmed(), |s| s == "0:00:00")
+                .color_if(Style::new().bold(), |s| s != "0:00:00"),
 
             // today
-            Col::min_width(9).and_alignment(Right).color_if(Style::new().dimmed(), |s| s == "0:00:00"),
+            Col::min_width(9).and_alignment(Right)
+                .color_if(Style::new().dimmed(), |s| s == "0:00:00")
+                .color_if(Style::new().bold(), |s| s != "0:00:00"),
 
             // accumulated
             Col::min_width(12).and_alignment(Right),
@@ -139,9 +144,9 @@ impl<'a> Command<'a> for ListCommand {
             format_duration(total),
         ]);
 
-        out.write_all(tabs.print().as_bytes())?;
+        streams.out.write_all(tabs.print(facts.env.stdout_is_tty).as_bytes())?;
 
-        warn_if_needed(err, needs_warning)?;
+        warn_if_needed(&mut streams.err, needs_warning, &facts.env)?;
 
         Ok(())
     }
@@ -151,7 +156,6 @@ impl<'a> Command<'a> for ListCommand {
 mod tests {
     use chrono::{Utc, TimeZone};
     use pretty_assertions::assert_eq;
-    use ansi_term::{Color::Yellow, Style};
 
     use crate::database::{SqliteDatabase, Database};
     use crate::test_utils::Ps;
@@ -163,68 +167,66 @@ mod tests {
         std::env::set_var("TZ", "CST+6");
 
         let args = Default::default();
-        let mut db = SqliteDatabase::from_memory().unwrap();
-        let mut out = Vec::new();
-        let mut err = Vec::new();
-        let config = Default::default();
-        db.init().unwrap();
-        db.set_current_sheet("sheet2").unwrap();
-        db.set_last_sheet("sheet4").unwrap();
+        let mut streams = Streams::fake(b"");
 
-        db.entry_insert(Utc.ymd(2021, 1, 1).and_hms(0, 0, 0), Some(Utc.ymd(2021, 1, 1).and_hms(1, 0, 0)), None, "_archived".into()).unwrap();
-        db.entry_insert(Utc.ymd(2021, 1, 1).and_hms(0, 0, 0), Some(Utc.ymd(2021, 1, 1).and_hms(10,13, 55)), None, "sheet1".into()).unwrap();
-        db.entry_insert(Utc.ymd(2021, 1, 1).and_hms(0, 0, 0), Some(Utc.ymd(2021, 1, 1).and_hms(7, 39, 18)), None, "sheet3".into()).unwrap();
-        db.entry_insert(Utc.ymd(2021, 1, 1).and_hms(12, 0, 0), Some(Utc.ymd(2021, 1, 1).and_hms(13, 52, 45)), None, "sheet3".into()).unwrap();
-        db.entry_insert(Utc.ymd(2021, 1, 1).and_hms(12, 0, 0), None, None, "sheet4".into()).unwrap();
+        streams.db.set_current_sheet("sheet2").unwrap();
+        streams.db.set_last_sheet("sheet4").unwrap();
+
+        streams.db.entry_insert(Utc.ymd(2021, 1, 1).and_hms(0, 0, 0), Some(Utc.ymd(2021, 1, 1).and_hms(1, 0, 0)), None, "_archived".into()).unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 1, 1).and_hms(0, 0, 0), Some(Utc.ymd(2021, 1, 1).and_hms(10,13, 55)), None, "sheet1".into()).unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 1, 1).and_hms(0, 0, 0), Some(Utc.ymd(2021, 1, 1).and_hms(7, 39, 18)), None, "sheet3".into()).unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 1, 1).and_hms(12, 0, 0), Some(Utc.ymd(2021, 1, 1).and_hms(13, 52, 45)), None, "sheet3".into()).unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 1, 1).and_hms(12, 0, 0), None, None, "sheet4".into()).unwrap();
 
         let now = Utc.ymd(2021, 1, 1).and_hms(13, 52, 45);
+        let facts = Facts::new().with_now(now);
 
-        ListCommand::handle(args, &mut db, &mut out, &mut err, &config, now).unwrap();
+        ListCommand::handle(args, &mut streams, &facts).unwrap();
 
-        assert_eq!(Ps(&String::from_utf8_lossy(&out)), Ps(&format!("  Timesheet   Running     Today   Total Time
+        assert_eq!(Ps(&String::from_utf8_lossy(&streams.out)), Ps("  Timesheet   Running     Today   Total Time
 
-  sheet1    {0} {0}     10:13:55
-* sheet2    {0} {0}      0:00:00
-  sheet3    {0}   1:52:45      9:32:03
+  sheet1      0:00:00   0:00:00     10:13:55
+* sheet2      0:00:00   0:00:00      0:00:00
+  sheet3      0:00:00   1:52:45      9:32:03
 - sheet4      1:52:45   1:52:45      1:52:45
 --------------------------------------------
               1:52:45   3:45:30     21:38:43
-", Style::new().dimmed().paint("  0:00:00"))));
+"));
 
         // now show all the sheets
-        let mut out = Vec::new();
-        let mut err = Vec::new();
+        streams.reset_io();
+
         let args = Args {
             all: true,
         };
 
-        ListCommand::handle(args, &mut db, &mut out, &mut err, &config, now).unwrap();
+        ListCommand::handle(args, &mut streams, &facts).unwrap();
 
-        assert_eq!(Ps(&String::from_utf8_lossy(&out)), Ps(&format!("  Timesheet   Running     Today   Total Time
+        assert_eq!(Ps(&String::from_utf8_lossy(&streams.out)), Ps("  Timesheet   Running     Today   Total Time
 
-  _archived {0} {0}      1:00:00
-  sheet1    {0} {0}     10:13:55
-* sheet2    {0} {0}      0:00:00
-  sheet3    {0}   1:52:45      9:32:03
+  _archived   0:00:00   0:00:00      1:00:00
+  sheet1      0:00:00   0:00:00     10:13:55
+* sheet2      0:00:00   0:00:00      0:00:00
+  sheet3      0:00:00   1:52:45      9:32:03
 - sheet4      1:52:45   1:52:45      1:52:45
 --------------------------------------------
               1:52:45   3:45:30     22:38:43
-", Style::new().dimmed().paint("  0:00:00"))));
+"));
     }
 
     #[test]
     fn old_database() {
         let args = Default::default();
-        let mut db = SqliteDatabase::from_path("assets/test_list_old_database.db").unwrap();
-        let mut out = Vec::new();
-        let mut err = Vec::new();
-        let config = Default::default();
+        let mut streams = Streams::fake(b"").with_db(
+            SqliteDatabase::from_path("assets/test_list_old_database.db").unwrap()
+        );
 
         let now = Local.ymd(2021, 7, 16).and_hms(11, 30, 45);
+        let facts = Facts::new().with_now(now.with_timezone(&Utc));
 
-        ListCommand::handle(args, &mut db, &mut out, &mut err, &config, now.with_timezone(&Utc)).unwrap();
+        ListCommand::handle(args, &mut streams, &facts).unwrap();
 
-        assert_eq!(Ps(&String::from_utf8_lossy(&out)), Ps("  Timesheet   Running     Today   Total Time
+        assert_eq!(Ps(&String::from_utf8_lossy(&streams.out)), Ps("  Timesheet   Running     Today   Total Time
 
 * default     0:10:24   0:10:26      0:10:26
 --------------------------------------------
@@ -232,8 +234,8 @@ mod tests {
 "));
 
         assert_eq!(
-            String::from_utf8_lossy(&err),
-            format!("{} You are using the old timetrap format, it is advised that you update your database using t migrate\n", Yellow.bold().paint("[WARNING]")),
+            String::from_utf8_lossy(&streams.err),
+            "[WARNING] You are using the old timetrap format, it is advised that you update your database using t migrate\n"
         );
     }
 }
