@@ -1,5 +1,5 @@
 use std::convert::TryFrom;
-use std::io::Write;
+use std::io::{BufRead, Write};
 use std::str::FromStr;
 
 use clap::ArgMatches;
@@ -9,10 +9,10 @@ use regex::Regex;
 use crate::error::{Result, Error};
 use crate::database::Database;
 use crate::formatters::Formatter;
-use crate::config::Config;
 use crate::regex::parse_regex;
+use crate::io::Streams;
 
-use super::{Command, display::{Sheet, entries_for_display}};
+use super::{Command, Facts, display::{Sheet, entries_for_display}};
 
 /// Given a local datetime, returns the time when the month it belongs started
 fn beginning_of_month(time: DateTime<Local>) -> DateTime<Utc> {
@@ -93,12 +93,14 @@ pub struct MonthCommand { }
 impl<'a> Command<'a> for MonthCommand {
     type Args = Args;
 
-    fn handle<D, O, E>(args: Self::Args, db: &mut D, out: &mut O, err: &mut E, config: &Config, now: DateTime<Utc>) -> Result<()>
+    fn handle<D, I, O, E>(args: Self::Args, streams: &mut Streams<D, I, O, E>, facts: &Facts) -> Result<()>
     where
         D: Database,
+        I: BufRead,
         O: Write,
         E: Write,
     {
+        let now = facts.now;
         let (start, end) = match args.month {
             MonthSpec::This => (beginning_of_month(now.with_timezone(&Local)), now),
             MonthSpec::Last => {
@@ -129,13 +131,22 @@ impl<'a> Command<'a> for MonthCommand {
             },
         };
 
-        entries_for_display(Some(start), Some(end), args.sheet, db, out, err, args.format.unwrap_or_else(|| config.default_formatter.clone()), args.ids, args.grep, now)
+        entries_for_display(
+            Some(start),
+            Some(end),
+            args.sheet,
+            streams,
+            args.format.unwrap_or_else(|| facts.config.default_formatter.clone()),
+            args.ids,
+            args.grep,
+            facts
+        )
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use crate::database::SqliteDatabase;
+    use crate::config::Config;
 
     use super::*;
 
@@ -144,22 +155,19 @@ mod tests {
         std::env::set_var("TZ", "CST+6");
 
         let args = Default::default();
-        let mut db = SqliteDatabase::from_memory().unwrap();
-        let mut out = Vec::new();
-        let mut err = Vec::new();
-        let config = Config {
+        let mut streams = Streams::fake(b"");
+        let now = Utc.ymd(2021, 6, 30).and_hms(11, 0, 0);
+        let facts = Facts::new().with_config(Config {
             default_formatter: Formatter::Ids,
             ..Default::default()
-        };
+        }).with_now(now);
 
-        db.init().unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 0, 0), None, Some("hola".into()), "default".into()).unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 10, 0), None, Some("hola".into()), "default".into()).unwrap();
 
-        db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 0, 0), None, Some("hola".into()), "default".into()).unwrap();
-        db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 10, 0), None, Some("hola".into()), "default".into()).unwrap();
+        MonthCommand::handle(args, &mut streams, &facts).unwrap();
 
-        MonthCommand::handle(args, &mut db, &mut out, &mut err, &config, Utc.ymd(2021, 6, 30).and_hms(11, 0, 0)).unwrap();
-
-        assert_eq!(&String::from_utf8_lossy(&out), "1 2\n");
-        assert_eq!(String::from_utf8_lossy(&err), "");
+        assert_eq!(&String::from_utf8_lossy(&streams.out), "1 2\n");
+        assert_eq!(String::from_utf8_lossy(&streams.err), "");
     }
 }
