@@ -1,5 +1,5 @@
 use std::convert::TryFrom;
-use std::io::Write;
+use std::io::{BufRead, Write};
 
 use clap::ArgMatches;
 use chrono::{DateTime, Utc, Local, Duration, Weekday, Datelike};
@@ -8,11 +8,12 @@ use regex::Regex;
 use crate::error::{Result, Error};
 use crate::database::Database;
 use crate::formatters::Formatter;
-use crate::config::{Config, WeekDay};
+use crate::config::WeekDay;
 use crate::regex::parse_regex;
 use crate::timeparse::parse_time;
+use crate::io::Streams;
 
-use super::{Command, display::{Sheet, entries_for_display}};
+use super::{Command, Facts, display::{Sheet, entries_for_display}};
 
 trait AsNum {
     fn as_num(&self) -> i64;
@@ -86,15 +87,25 @@ pub struct WeekCommand { }
 impl<'a> Command<'a> for WeekCommand {
     type Args = Args;
 
-    fn handle<D, O, E>(args: Self::Args, db: &mut D, out: &mut O, err: &mut E, config: &Config, now: DateTime<Utc>) -> Result<()>
+    fn handle<D, I, O, E>(args: Self::Args, streams: &mut Streams<D, I, O, E>, facts: &Facts) -> Result<()>
     where
         D: Database,
+        I: BufRead,
         O: Write,
         E: Write,
     {
-        let start = prev_day(now.with_timezone(&Local), config.week_start);
+        let start = prev_day(facts.now.with_timezone(&Local), facts.config.week_start);
 
-        entries_for_display(Some(start), args.end, args.sheet, db, out, err, args.format.unwrap_or_else(|| config.default_formatter.clone()), args.ids, args.grep, now)
+        entries_for_display(
+            Some(start),
+            args.end,
+            args.sheet,
+            streams,
+            args.format.unwrap_or_else(|| facts.config.default_formatter.clone()),
+            args.ids,
+            args.grep,
+            facts
+        )
     }
 }
 
@@ -102,7 +113,7 @@ impl<'a> Command<'a> for WeekCommand {
 mod tests {
     use chrono::TimeZone;
 
-    use crate::database::SqliteDatabase;
+    use crate::config::Config;
 
     use super::*;
 
@@ -125,22 +136,19 @@ mod tests {
         std::env::set_var("TZ", "CST+6");
 
         let args = Default::default();
-        let mut db = SqliteDatabase::from_memory().unwrap();
-        let mut out = Vec::new();
-        let mut err = Vec::new();
-        let config = Config {
+        let mut streams = Streams::fake(b"");
+        let now = Utc.ymd(2021, 7, 1).and_hms(10, 0, 0);
+        let facts = Facts::new().with_config(Config {
             default_formatter: Formatter::Ids,
             ..Default::default()
-        };
+        }).with_now(now);
 
-        db.init().unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 0, 0), None, Some("hola".into()), "default".into()).unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 10, 0), None, Some("hola".into()), "default".into()).unwrap();
 
-        db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 0, 0), None, Some("hola".into()), "default".into()).unwrap();
-        db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 10, 0), None, Some("hola".into()), "default".into()).unwrap();
+        WeekCommand::handle(args, &mut streams, &facts).unwrap();
 
-        WeekCommand::handle(args, &mut db, &mut out, &mut err, &config, Utc.ymd(2021, 7, 1).and_hms(10, 0, 0)).unwrap();
-
-        assert_eq!(&String::from_utf8_lossy(&out), "1 2\n");
-        assert_eq!(String::from_utf8_lossy(&err), "");
+        assert_eq!(&String::from_utf8_lossy(&streams.out), "1 2\n");
+        assert_eq!(String::from_utf8_lossy(&streams.err), "");
     }
 }
