@@ -1,17 +1,17 @@
 use std::convert::TryFrom;
-use std::io::Write;
+use std::io::{BufRead, Write};
 
 use clap::ArgMatches;
-use chrono::{DateTime, Utc, Local, Duration};
+use chrono::{Utc, Local, Duration};
 use regex::Regex;
 
 use crate::error::{Result, Error};
 use crate::database::Database;
 use crate::formatters::Formatter;
-use crate::config::Config;
 use crate::regex::parse_regex;
+use crate::io::Streams;
 
-use super::{Command, display::{Sheet, entries_for_display}};
+use super::{Command, Facts, display::{Sheet, entries_for_display}};
 
 #[derive(Default)]
 pub struct Args {
@@ -39,17 +39,27 @@ pub struct YesterdayCommand { }
 impl<'a> Command<'a> for YesterdayCommand {
     type Args = Args;
 
-    fn handle<D, O, E>(args: Self::Args, db: &mut D, out: &mut O, err: &mut E, config: &Config, now: DateTime<Utc>) -> Result<()>
+    fn handle<D, I, O, E>(args: Self::Args, streams: &mut Streams<D, I, O, E>, facts: &Facts) -> Result<()>
     where
         D: Database,
+        I: BufRead,
         O: Write,
         E: Write,
     {
-        let today = now.with_timezone(&Local).date();
+        let today = facts.now.with_timezone(&Local).date();
         let start = Some((today - Duration::days(1)).and_hms(0, 0, 0).with_timezone(&Utc));
         let end = Some(today.and_hms(0, 0, 0).with_timezone(&Utc));
 
-        entries_for_display(start, end, args.sheet, db, out, err, args.format.unwrap_or_else(|| config.default_formatter.clone()), args.ids, args.grep, now)
+        entries_for_display(
+            start,
+            end,
+            args.sheet,
+            streams,
+            args.format.unwrap_or_else(|| facts.config.default_formatter.clone()),
+            args.ids,
+            args.grep,
+            facts
+        )
     }
 }
 
@@ -58,8 +68,8 @@ mod tests {
     use chrono::{Duration, TimeZone};
     use pretty_assertions::assert_eq;
 
-    use crate::database::SqliteDatabase;
     use crate::test_utils::Ps;
+    use crate::config::Config;
 
     use super::*;
 
@@ -69,29 +79,24 @@ mod tests {
             format: Some(Formatter::Csv),
             ..Default::default()
         };
-        let mut db = SqliteDatabase::from_memory().unwrap();
-        let mut out = Vec::new();
-        let mut err = Vec::new();
-        let config = Default::default();
-
-        db.init().unwrap();
-
+        let mut streams = Streams::fake(b"");
         let two_days_ago = Local::now().date() - Duration::days(2);
         let yesterday = Local::now().date() - Duration::days(1);
         let today = Local::now().date();
+        let facts = Facts::new();
 
-        db.entry_insert(two_days_ago.and_hms(1, 2, 3).with_timezone(&Utc), None, None, "default".into()).unwrap();
-        db.entry_insert(yesterday.and_hms(1, 2, 3).with_timezone(&Utc), None, Some("This!".into()), "default".into()).unwrap();
-        db.entry_insert(today.and_hms(1, 2, 3).with_timezone(&Utc), None, None, "default".into()).unwrap();
+        streams.db.entry_insert(two_days_ago.and_hms(1, 2, 3).with_timezone(&Utc), None, None, "default".into()).unwrap();
+        streams.db.entry_insert(yesterday.and_hms(1, 2, 3).with_timezone(&Utc), None, Some("This!".into()), "default".into()).unwrap();
+        streams.db.entry_insert(today.and_hms(1, 2, 3).with_timezone(&Utc), None, None, "default".into()).unwrap();
 
-        YesterdayCommand::handle(args, &mut db, &mut out, &mut err, &config, Utc::now()).unwrap();
+        YesterdayCommand::handle(args, &mut streams, &facts).unwrap();
 
-        assert_eq!(Ps(&String::from_utf8_lossy(&out)), Ps(&format!("start,end,note,sheet
+        assert_eq!(Ps(&String::from_utf8_lossy(&streams.out)), Ps(&format!("start,end,note,sheet
 {},,This!,default
 ", yesterday.and_hms(1, 2, 3).with_timezone(&Utc).to_rfc3339_opts(chrono::SecondsFormat::Micros, true))));
 
         assert_eq!(
-            String::from_utf8_lossy(&err),
+            String::from_utf8_lossy(&streams.err),
             String::new(),
         );
     }
@@ -101,22 +106,19 @@ mod tests {
         std::env::set_var("TZ", "CST+6");
 
         let args = Default::default();
-        let mut db = SqliteDatabase::from_memory().unwrap();
-        let mut out = Vec::new();
-        let mut err = Vec::new();
-        let config = Config {
+        let mut streams = Streams::fake(b"");
+        let now = Utc.ymd(2021, 7, 1).and_hms(10, 0, 0);
+        let facts = Facts::new().with_config(Config {
             default_formatter: Formatter::Ids,
             ..Default::default()
-        };
+        }).with_now(now);
 
-        db.init().unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 0, 0), None, Some("hola".into()), "default".into()).unwrap();
+        streams.db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 10, 0), None, Some("hola".into()), "default".into()).unwrap();
 
-        db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 0, 0), None, Some("hola".into()), "default".into()).unwrap();
-        db.entry_insert(Utc.ymd(2021, 6, 30).and_hms(10, 10, 0), None, Some("hola".into()), "default".into()).unwrap();
+        YesterdayCommand::handle(args, &mut streams, &facts).unwrap();
 
-        YesterdayCommand::handle(args, &mut db, &mut out, &mut err, &config, Utc.ymd(2021, 7, 1).and_hms(10, 0, 0)).unwrap();
-
-        assert_eq!(&String::from_utf8_lossy(&out), "1 2\n");
-        assert_eq!(String::from_utf8_lossy(&err), "");
+        assert_eq!(&String::from_utf8_lossy(&streams.out), "1 2\n");
+        assert_eq!(String::from_utf8_lossy(&streams.err), "");
     }
 }
