@@ -246,19 +246,19 @@ pub struct SqliteDatabase {
 }
 
 impl SqliteDatabase {
-    pub fn from_memory() -> Result<impl Database> {
+    pub fn from_memory() -> Result<SqliteDatabase> {
         Ok(SqliteDatabase {
             connection: Connection::open_in_memory()?,
         })
     }
 
-    pub fn from_path<P: AsRef<Path>>(path: P) -> Result<impl Database> {
+    pub fn from_path<P: AsRef<Path>>(path: P) -> Result<SqliteDatabase> {
         Ok(SqliteDatabase {
             connection: Connection::open(path)?,
         })
     }
 
-    pub fn from_path_or_create<P: AsRef<Path>>(path: P) -> Result<impl Database> {
+    pub fn from_path_or_create<P: AsRef<Path>>(path: P) -> Result<SqliteDatabase> {
         if path.as_ref().is_file() {
             Self::from_path(path)
         } else {
