@@ -13,6 +13,8 @@ pub mod regex;
 pub mod tabulate;
 pub mod old;
 pub mod interactive;
+pub mod env;
+pub mod io;
 
 #[cfg(test)]
 pub mod test_utils;
